@@ -222,7 +222,7 @@ class LCodeGen BASE_EMBEDDED {
   Register ToRegister(int index) const;
   XMMRegister ToDoubleRegister(int index) const;
   int ToInteger32(LConstantOperand* op) const;
-  Operand BuildFastArrayOperand(LOperand* external_pointer,
+  Operand BuildFastArrayOperand(LOperand* elements_pointer,
                                 LOperand* key,
                                 JSObject::ElementsKind elements_kind,
                                 uint32_t offset);
